# coding:utf-8
from tools_hjh.HTTPRequest import HTTPRequest
from math import ceil
from tools_hjh.Tools import mkdir, rm
from tools_hjh.ThreadPool import ThreadPool
import os

    
def main():
    headers = {
        'cookie':'cf_clearance=ZyXhlDX.MEsAT8edtS9fzDZpnwDpDBgN8jzhq4uzQHY-1714422048-1.0.1.1-gMVkjl718OqgU4CMIWop9tl4WCWpgYdT3aizn70Z0z905jYV.zSYdAwMg1OScze8BsMUQtkcdNJ3GtfT0m3ZMA; csrftoken=ZgclpVKZTiRpWHRhF0OOeTdSlU5odAJsyWq2a37iNNizZykV4w2XAhL9VOWW7hIT',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-language': 'zh-CN,zh;q=0.9',
        'cache-control': 'no-cache',
        'cookie':'',
        'dnt': '1',
        'pragma': 'no-cache',
        'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'Referer': 'https://avbebe.com/',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36'
    }
    # size = HTTPTools.get_size(url='https://v.avgigi.com/acg/watch/198910.mp4', headers=headers)
    # size = HTTPTools.download(url='https://www.bixiashuku.com/modules/article/txtarticle.php?id=87850', filepath='D:/1.txt', headers=headers, retry_until_success=True)
    size = HTTPTools.get(url='https://nhentai.net/')
    print(size)


class HTTPTools():
    
    @staticmethod
    def get_page(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        text = ''
        if retry_until_success:
            try:
                req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
                text = req.get_text()
                req.close()
            except:
                HTTPTools.get_page(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success)
        else: 
            req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
            text = req.get_text()
            req.close()
        return text

    @staticmethod
    def get_content(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        content = ''
        if retry_until_success:
            try:
                req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
                content = req.get_content()
                req.close()
            except:
                HTTPTools.get_content(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success)
        else: 
            req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
            content = req.get_content()
            req.close()
        return content
    
    @staticmethod
    def get(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        return HTTPTools.get_page(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success)
    
    @staticmethod
    def get_size(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        text = 0
        if retry_until_success:
            try:
                req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
                text = req.get_size()
                req.close()
            except:
                HTTPTools.get_page(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success)
        else: 
            req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
            text = req.get_size()
            req.close()
        return text
    
    @staticmethod
    def get_status_code(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        text = ''
        if retry_until_success:
            try:
                req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
                text = req.get_status_code()
                req.close()
            except:
                HTTPTools.get_page(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success)
        else: 
            req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
            text = req.get_status_code()
            req.close()
        return text

    @staticmethod
    def download(url, filepath, replace=False, partsize=1024 * 1024, threadnum=1, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False, retry_until_success=False):
        req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
        if threadnum == 1:
            if not retry_until_success:
                return req.download(filepath=filepath, replace=replace)
            else:
                return req.download_until_success(filepath=filepath, replace=replace)
        else:
            # 有问题，还没看出问题出在哪里QAQ
            tp = ThreadPool(threadnum)
            filesize = HTTPTools.get_size(url, headers, data, proxies, encoding, timeout, stream, allow_redirects, verify, retry_until_success=False)
            partnum = ceil(filesize / partsize)
            filepath = filepath.replace('\\', '/')
            path = filepath.rsplit('/', 1)[0] + '/'
            name = filepath.split('/')[-1]
            tmp_path = path + name + '_tmp/'
            if replace:
                rm(filepath)
            mkdir(tmp_path)
            downloaded_files = []
            for idx in range(partnum):
                begin = idx * partsize + 1
                if idx == partnum - 1:
                    end = filesize
                else:
                    end = begin + partsize - 1
                new_filepath = tmp_path + str(idx)
                downloaded_files.append(new_filepath)
                new_headers = headers.copy()
                new_headers['Range'] = f'bytes={begin}-{end}'
                if retry_until_success:
                    tp.run(HTTPTools.download, (url, new_filepath, True, 1024 * 1024, 1, new_headers, data, proxies, encoding, timeout, stream, allow_redirects, verify))
                else:
                    tp.run(HTTPTools.download_until_success, (url, new_filepath, True, 1024 * 1024, 1, new_headers, data, proxies, encoding, timeout, stream, allow_redirects, verify))
            tp.wait()
            
            size = 0
            filepath = open(filepath, 'wb')
            for downloaded_file in downloaded_files:
                if os.path.exists(downloaded_file):
                    f = open(downloaded_file, "rb")
                    tsfileRead = f.read()
                size = size + filepath.write(tsfileRead)
                f.close()
            filepath.close()
            rm(tmp_path)
            return size
        
    @staticmethod
    def N_m3u8DL_CLI(url, filepath, filename, theadnum=32, exe_path='N_m3u8DL-CLI_v3.0.2.exe', headers=''):
        cmd = exe_path + ' --workDir ' + filepath + ' --saveName "' + filename + '" --maxThreads ' + str(theadnum) + ' --enableDelAfterDone ' + '--headers "' + headers + '" ' + url
        os.popen('chcp 65001')
        os.system(cmd)

    
if __name__ == '__main__':
    main()



# coding:utf-8
from uuid import uuid1
from concurrent.futures import ThreadPoolExecutor
import time


def main():
    pass


class ThreadPool():
    """ 维护一个线程池 """
    
    def __init__(self, size, while_wait_time=0.1):
        self.tid_task = {}
        self.tid_status = {}
        self.tid_trs = {}
        self.tid_tname = {}
        self.size = size
        self.while_wait_time = while_wait_time
        self.pool = ThreadPoolExecutor(max_workers=self.size)
        
    def run_nowait(self, func, args, kwargs={}, name=None):
        thread_id = uuid1()
        task = self.pool.submit(func, *args, **kwargs)
        self.tid_task[thread_id] = task
        self.tid_tname[thread_id] = name
        return thread_id

    def run(self, func, args, kwargs={}, name=None):
        """ 主线程命令当前线程池从空闲线程中取一个线程执行给入的方法，如果池满，则主线程等待 """
        while self.get_running_num() >= self.size:
            time.sleep(self.while_wait_time)
        return self.run_nowait(func=func, args=args, kwargs=kwargs, name=name)

    def get_results(self):
        self.wait()
        return self.tid_trs
    
    def get_result(self, thread_id):
        self.get_results()
        return self.tid_trs[thread_id]

    def wait(self):
        """ 主线程等待，直到线程池不存在活动线程 """
        for tid, task in self.tid_task.items():
            if tid not in self.tid_trs:
                self.tid_trs[tid] = task.result()
    
    def get_running_num(self):
        running_thread_ids = self.get_running_id()
        return len(running_thread_ids)
    
    def get_running_id(self):
        running_thread_ids = []
        for tid, task in self.tid_task.copy().items():
            if not task.done():
                running_thread_ids.append(tid)
        return running_thread_ids
    
    def get_running_name(self):
        running_names = []
        running_thread_ids = self.get_running_id()
        for tid in running_thread_ids:
            running_names.append(self.tid_tname[tid])
        return running_names
    
    def clear(self):
        self.tid_task = {}
        self.tid_status = {}
        self.tid_trs = {}
        self.tid_tname = {}


if __name__ == '__main__':
    main()



# coding:utf-8
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import time
from tools_hjh.ThreadPool import ThreadPool
import os
from tools_hjh import Tools


def main():
    tp = ThreadPool(2)
    chrome_path = r'U:\MyApps\CentBrowser\App\chrome.exe'
    chromedriver_path = r'U:\MyApps\CentBrowser\chromedriver.exe'

    cp = ChromePool(2, chrome_path=chrome_path, chromedriver_path=chromedriver_path)
    u1 = tp.run(cp.get, ('https://www.baidu.com',))
    time.sleep(1)
    u2 = tp.run(cp.get, ('https://www.yeah.net',))
    
    tp.wait()
    print(len(tp.get_result(u1)), len(tp.get_result(u2)))
    cp.close()


class Chrome():
    """ 使用浏览器解析url，返回源码
        __init__.param：
            chrome_path: chrome.exe路径
            chromedriver_path: chromedriver.exe路径
    """

    def __init__(self, chrome_path, chromedriver_path, is_hidden=False, is_display_picture=True, headers=None):
        chrome_options = Options()
        # chrome_options.add_argument("--single-process")
        # chrome_options.add_argument("--disable-site-isolation-trials")

        if is_hidden:
            chrome_options.add_argument("--headless")
        chrome_options.binary_location = chrome_path
        if not is_display_picture:
            chrome_options.add_experimental_option('prefs', {'profile.managed_default_content_settings.images': 2})
        if headers is not None:
            for key, value in headers.items():
                header = key + "=" + value
                chrome_options.add_argument(header)
        self.chrome = webdriver.Chrome(chromedriver_path, options=chrome_options)
        self.status = 0  # 未被占用
    
    def set_cookies(self, cookies, cookie_domian):
        self.get(cookie_domian)
        for cookie in cookies:
            self.chrome.add_cookie(cookie)
        
    def close(self):
        self.chrome.quit()
        
    def get(self, url):
        self.status = 1  # 被占用
        self.chrome.get(url)
        text = self.chrome.page_source
        self.status = 0
        return text
    
    def get_status(self):
        return self.status


class ChromePool():

    def __init__(self, maxSize, chrome_path, chromedriver_path, is_hidden=False, is_display_picture=True, cache=False, headers=None):
        self.cache = cache
        self.maxSize = maxSize
        self.pool = []
        for _ in range(0, maxSize):
            self.pool.append(Chrome(chrome_path, chromedriver_path, is_hidden, is_display_picture, headers))
        self.openSize = 0
        
    def set_cookies(self, cookies, cookie_domian):

        def set_one(chrome, cookies, cookie_domian):
            chrome.set_cookies(cookies, cookie_domian)

        tp = ThreadPool(self.maxSize)
        for chrome in self.pool:
            tp.run(set_one, (chrome, cookies, cookie_domian))
        tp.wait()
        
    def get(self, url):
        while self.openSize == self.maxSize:
            time.sleep(0.2)
        for chrome in self.pool:
            if chrome.get_status() == 0:
                self.openSize = self.openSize + 1
                if self.cache:
                    url_split = url.split('/')
                    host = url_split[0] + '//' + url_split[2] + '/'
                    path_ = 'tmp/' + url.replace(host, '').rstrip('/')
                    if os.path.exists(path_):
                        text = Tools.cat(path_)
                    else:
                        text = chrome.get(url)
                        Tools.echo(text, path_)
                else:
                    text = chrome.get(url)
                self.openSize = self.openSize - 1
                return text
        time.sleep(0.2)
        return self.get(url)
    
    def close(self):
        while self.openSize > 0:
            time.sleep(0.2)
        for c in self.pool:
            c.close()


if __name__ == '__main__':
    main()



# coding:utf-8
import sys
import gc
from tools_hjh import DBConn
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from tools_hjh import HTTPTools
from bs4 import BeautifulSoup

stop_flag = False
page_num = None

date = Tools.locatdate()
log = Log('U:/MyFiles/MyPy/log/mmys/' + date + '.log')

host = 'https://www.mmys07.one'

chrome_path = r'U:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'U:\MyApps\CentBrowser\chromedriver.exe'

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 32
    

def main(): 
    db = DBConn('sqlite', db='U:/MyFiles/MyPy/data/mmys.db')
    createDatabase(db, rebuild=False)
    # get_main(db, '/category/3/8/page/PAGEIDX', 'youxi', HTTPTools)
    # get_main(db, '/category/3/10/page/PAGEIDX', 'donghua', HTTPTools)
    # get_main(db, '/category/3/4/page/PAGEIDX', 'lifan', HTTPTools)
    # get_main(db, '/category/3/5/page/PAGEIDX', 'shengyin', HTTPTools)
    get_youxi(db)
    # fenxi(db)

    
def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')
        db.run('drop table if exists t_youxi')

    t_m = '''
        create table if not exists t_m(
            url text, 
            name text,
            uploader text,
            type text
        )
    '''
    
    t_youxi = '''
        create table if not exists t_youxi(
            url text, 
            name text,
            shetuan text,
            riqi text,
            gengxin text,
            nianling text,
            leixing text,
            xingshi text,
            yuyan text,
            biaoqian text,
            daxiao text,
            xilie text,
            zuozhe text,
            juqing text,
            chahua text,
            shengyou text,
            yinyue text,
            qita text,
            message text
        )
    '''
    
    db.run(t_m)
    db.run(t_youxi)
    db.run('create index if not exists idx_m_url on t_m(url)')
    db.run('create index if not exists idx_yx_url on t_youxi(url)')
    db.run('create index if not exists idx_yx_name on t_youxi(name)')

    
def get_main(db, firstPageUrl, type_, chrome):

    def get_main_one(idx, type_, db, times=1):
        pageurl = host + firstPageUrl.replace('PAGEIDX', str(idx))
        
        url = ''
        name = ''
        uploader = ''
        
        try:
            page = HTTPTools.get(pageurl)
            bs = BeautifulSoup(page, features="lxml")
            divs = bs.find_all('div', class_='post-info')
            insert_params = []
            for div in divs:
                try:
                    url = div.find('h2').find('a')['href'].strip()
                    if host not in url:
                        log.warning('get_main_one', url, 'host not in url')
                        return
                    url = url.replace(host, '')
                    name = div.find('h2').find('a').text.strip()
                    uploader = div.find('a', class_='post-list-meta-avatar').find('span').text.strip()
                except Exception as _:
                    print(idx, _)
                
                insert_param = (url, name, uploader, type_, url)
                insert_params.append(insert_param)
            
            if len(insert_params) == 0 and idx < page_num:
                raise Exception("insert_params size == 0")
                
            insert_sql = 'insert into t_m select ?,?,?,? where not exists(select 1 from t_m t where t.url = ?)'
            insert_num = db.run(insert_sql, insert_params, wait=True)
            
            if insert_num == 0:
                global stop_flag
                stop_flag = True
            
            log.info('get_main_one', pageurl, insert_num, len(insert_params), tp.get_running_num(), times)
        except Exception as _:
            if times <= 10:
                times = times + 1 
                get_main_one(idx, type_, db, times)
            else:
                log.error('get_main_one', pageurl, times, _)
    
    tp = ThreadPool(sys_argv_3)
    global page_num
    global stop_flag
    stop_flag = False
    if page_num is None:
        pageurl = host + firstPageUrl.replace('PAGEIDX', '1')
        page = chrome.get(pageurl)
        bs = BeautifulSoup(page, features="lxml")
        page_num = int(bs.find('div', class_='b2-pagenav post-nav box mg-t b2-radius')['data-max'])
    for idx in range(1, page_num + 1):
    # for idx in range(1394, 1401 + 1):
        if stop_flag:
            break
        else:
            tp.run(get_main_one, (idx, type_, db))
    tp.wait()

    
def get_youxi(db):
    get_url_sql = '''
        select url from t_m 
        where not exists(select 1 from t_youxi where t_m.url = t_youxi.url) 
        --and exists(select 1 from t_page where t_page.url = t_m.url)
        and (
            name like '[同人游戏]%'
            or name like '[手机游戏]%'
            or name like '[Gal]%'
        ) order by cast(replace(url,'https://www.mmys04.one/archives/','') as int) desc
    '''
    # get_page_sql = "select page from t_page where url = ?"
    insert_sql = "insert into t_youxi select ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,? where not exists(select 1 from t_youxi where url = ?)"
    params = []
    
    def get_youxi_one(url, rep):
        
        def get_td(tr):
            td = ''
            if '<span>' in str(tr):
                spans = tr.find('td').find_all('span')
                for span in spans:
                    td = td + '丶' + span.text
            else:
                td = tr.find('td').text
            td = Tools.merge_spaces(td)
            td = td.replace('\n', ' ')
            td = td.replace('（', '(')
            td = td.replace('）', ')')
            td = td.strip('丶')
            td = td.strip()
            return td
        
        name = ''
        shetuan = ''
        riqi = ''
        gengxin = ''
        nianling = ''
        leixing = ''
        xingshi = ''
        yuyan = ''
        biaoqian = ''
        daxiao = ''
        xilie = ''
        zuozhe = ''
        juqing = ''
        chahua = ''
        shengyou = ''
        yinyue = ''
        qita = ''
        message = ''
                
        # page = db.run(get_page_sql, (url,)).get_rows()[0][0]
        page = rep.get(host + url)
        
        if '<title>WordPress &rsaquo; 错误</title>' in str(page):
            # db.run("delete from t_page where url = ?", (url,))
            log.error('get_youxi_one', url, 'page error')
            return
        
        bs = BeautifulSoup(page, features="lxml")
        try:
            if 'entry-header' in str(page):
                name = bs.find('header', class_='entry-header').find('h1').text.strip()
            else:
                name = bs.find('title').text.replace(' - 萌萌御所', '').strip()
        except:
            log.error('get_youxi_one', url, 'name is None')
            return
                
        try:
            message = bs.find('div', class_='content-excerpt').text.strip()
        except:
            pass
        
        if 'wp-block-table' not in str(page):
            log.error('get_youxi_one', url, 'table is None', name.split(']')[0] + ']')
            return
        
        table = bs.find('figure', class_='wp-block-table')
        tr_list = table.find_all('tr')
        for tr in tr_list:
            if '社团' in str(tr) or '商标' in str(tr):
                shetuan = get_td(tr)
            elif '日期' in str(tr):
                riqi = get_td(tr)
                riqi = riqi.split('日')[0].replace('年', '').replace('月', '')
            elif '更新' in str(tr):
                gengxin = get_td(tr)
                gengxin = gengxin.split('日')[0].replace('年', '').replace('月', '')
            elif '年龄' in str(tr):
                nianling = get_td(tr)
            elif '类型' in str(tr):
                leixing = get_td(tr)
            elif '形式' in str(tr):
                xingshi = get_td(tr)
            elif '语言' in str(tr):
                yuyan = get_td(tr)
            elif '标签' in str(tr):
                biaoqian = get_td(tr)
            elif '大小' in str(tr):
                daxiao = get_td(tr)
            elif '系列' in str(tr):
                xilie = get_td(tr)
            elif '作者' in str(tr):
                zuozhe = get_td(tr)
            elif '剧情' in str(tr):
                juqing = get_td(tr)
            elif '插画' in str(tr):
                chahua = get_td(tr)
            elif '声优' in str(tr):
                shengyou = get_td(tr)
            elif '音乐' in str(tr):
                yinyue = get_td(tr)
            elif '其他' in str(tr):
                qita = get_td(tr)
                
        if len(shetuan) > 0: 
            param = (url, name, shetuan, riqi, gengxin, nianling, leixing, xingshi, yuyan, biaoqian, daxiao, xilie, zuozhe, juqing, chahua, shengyou, yinyue, qita, message, url)        
            params.append(param)
            log.info('get_youxi_one', url, len(params), name.split(']')[0] + ']')
        else:
            log.warning('get_youxi_one', url, 'shetuan is null', name.split(']')[0] + ']')

    urls = db.run(get_url_sql).get_rows()
    tp = ThreadPool(32)
    for url_ in urls:
        if len(params) <= 200:
            url = url_[0]
            tp.run(get_youxi_one, (url, HTTPTools))
        else:
            tp.wait()
            num = db.run(insert_sql, params)
            log.info('get_youxi', 'insert', num)
            params.clear()
            gc.collect()
    tp.wait()
    num = db.run(insert_sql, params)
    log.info('get_youxi', 'insert', num)

    
def fenxi(db):
    get_url_sql = "select url from t_m where name like '[同人动画]%'"
    get_page_sql = "select page from t_page where url = ?"
    
    th_map = {}
    th_list = []
    
    def get_one(db, url, i):
        page = db.run(get_page_sql, (url,)).get_rows()[0][0]
        bs = BeautifulSoup(page, features="lxml")
        try:
            table = bs.find('figure', class_='wp-block-table')
            trs = table.find_all('tr')
        except Exception as _:
            log.error('fenxi', i, url, _)
            return
        
        for tr in trs:
            try:
                th = tr.find('th').text.strip()
                if th not in th_list:
                    th_list.append(th)
                    th_map[th] = 1
                else:
                    th_map[th] = th_map[th] + 1
            except Exception as _:
                pass
            
        log.info('fenxi', i, th_map)
    
    urls = db.run(get_url_sql).get_rows()
    tp = ThreadPool(64)
    i = 1
    for url_ in urls:
        url = url_[0]
        tp.run(get_one, (db, url, i), time_out=10)
        i = i + 1
    tp.wait()


def sql(db):
    sql = '''
        
    '''
    rss = db.run(sql).get_rows()
    for rs in rss:
        print(rs)

    
if __name__ == '__main__':
    main()

